unit FormMain;

{
    Copyright (c) 2004 headcrash industries. All rights reserved.

    Author: Gerke Preussner <j3rky@gerke-preussner.de>

    This file belongs to the InfoZIP UnZip for Delphi Wrapper. Please read
    the license agreement in the readme.txt that comes with this package.
    Latest updates can be found at http://www.gerke-preussner.de
}

interface

{*
========================
   includes
========================
*}

uses SysUtils, Classes, Controls, Forms, StdCtrls, Unzip;


type

{*
========================
   classes
========================
*}

   ////////////////////
   //  TForm_Main
   ////////////////////

   TForm_Main = class(TForm)

      Button_WizGrep: TButton;
      Button_WizInit: TButton;
      Button_WizSetOpts: TButton;
      Button_WizSingleEntryUnzip: TButton;
      Button_WizUnzip: TButton;
      Button_WizUnzipToMemory: TButton;
      Button_WizValidate: TButton;
      Button_UzpVersion: TButton;
      Button_UzpVersion2: TButton;

      Memo: TMemo;


      // app events
      procedure OnCreate_FormMain(Sender: TObject);

      // user events
      procedure OnClick_Button_WizGrep(Sender: TObject);
      procedure OnClick_Button_WizInit (Sender: TObject);
      procedure OnClick_Button_WizSetOpts (Sender: TObject);
      procedure OnClick_Button_WizSingleEntryUnzip (Sender: TObject);
      procedure OnClick_Button_WizUnzip (Sender: TObject);
      procedure OnClick_Button_WizUnzipToMemory (Sender: TObject);
      procedure OnClick_Button_WizValidate (Sender: TObject);
      procedure OnClick_Button_UzpVersion (Sender: TObject);
      procedure OnClick_Button_UzpVersion2 (Sender: TObject);

   private

      UFuncs: USERFUNCTIONS;
      DCList: DCL;

   end;


{*
========================
   callback functions
========================
*}

   // note: these are the callback routines that eventually get called by
   // the DLL. in order to work with unzip32.dll you need to specify at
   // least a callback routine for 'print' and 'replace' in the DCL record.

   procedure CallbackMessage (ucsize, csiz, cfactor, mo, dy, yr, hh, mm: Longword; c: Byte; fname, meth: PChar; crc: Longword; fCrypt: Byte); stdcall;
   function CallbackPassword (pwbuf: PChar; size: Longint; m, efn: PChar): EDllPassword; stdcall;
   function CallbackPrint (buffer: PChar; size: Longword): EDllPrint; stdcall;
   function CallbackReplace (filename: PChar): EDllReplace; stdcall;
   function CallbackService (efn: PChar; details: Longword): EDllService; stdcall;


{*
========================
   consts
========================
*}

const

   PATH_TO_ZIPFILE: PChar =     'testdata.zip';   // path to the sample archive
   PATH_TO_OUTPUTDIR: PChar =   'output\';        // path to the output directory
   SOME_ZIPPED_FILE: PChar =    'kakadu.txt';     // a file inside the sample archive
   SOME_GREP_STRING: PChar =    'philosoph';      // a string inside a file inside the sample archive


{*
========================
   globals
========================
*}

var
   Form_Main: TForm_Main;


implementation

uses FormVersion;

{$R *.dfm}

{*
========================
   TForm_Main (public)
========================
*}

////////////////////
//  app events
////////////////////

procedure TForm_Main.OnCreate_FormMain (Sender: TObject);
begin
   // note: the USERFUNCTIONS record contains pointers to your callback
   // functions. it is required for the DLL functions Wiz_Grep(),
   // Wiz_Init(), Wiz_SingleEntryUnzip() and Wiz_UnzipToMemory()

   with self.UFuncs do
      begin
      print := CallbackPrint;
      sound := nil;
      replace := CallbackReplace;
      password := CallbackPassword;
      SendApplicationMessage := CallbackMessage;
      ServCallBk := CallbackService;
      end;

   // note: the DCL record contains settings for the DLL. it is required
   // for the DLL functions Wiz_SetOpts() and Wiz_SingleEntryUnzip().
   // please check Unzip.pas for a description of the various options.

   with self.DCList do
      begin
      ExtractOnlyNewer := 0;
      SpaceToUnderscore := 0;
      PromptToOverwrite := 1;
      fQuiet := 0;
      ncflag := 0;
      ntflag := 0;
      nvflag := 0;
      nfflag := 0;
      nzflag := 0;
      ndflag := 1;
      noflag := 1;
      naflag := 0;
      nZIflag := 0;
      C_flag := 1;
      fPrivilege := 0;
      lpszZipFN := PATH_TO_ZIPFILE;
      lpszExtractDir := PATH_TO_OUTPUTDIR;
      end;
end;


////////////////////
//  user events
////////////////////

procedure TForm_Main.OnClick_Button_WizGrep (Sender: TObject);
begin
   // demo for Wiz_Grep()
   case Wiz_Grep(PATH_TO_ZIPFILE, SOME_ZIPPED_FILE, SOME_GREP_STRING, UZ_GREP_NORMAL, true, self.UFuncs) of
      -1:   self.Memo.Lines.Add('Wiz_Grep failed');
       0:   self.Memo.Lines.Add('The string ' + SOME_GREP_STRING + ' could not be found in ' + SOME_ZIPPED_FILE);
       1:   self.Memo.Lines.Add('The string ' + SOME_GREP_STRING + ' was found in ' + SOME_ZIPPED_FILE);
   end;
end;


procedure TForm_Main.OnClick_Button_WizInit (Sender: TObject);
var
   dummy:      Pointer;
begin
   // demo for Wiz_Init()
   if Wiz_Init(dummy, self.UFuncs) then
      self.Memo.Lines.Add('Wiz_Init succeeded')
   else
      self.Memo.Lines.Add('Error: Wiz_Init failed');
end;


procedure TForm_Main.OnClick_Button_WizSetOpts (Sender: TObject);
var
   dummy:      Pointer;
begin
   // demo for Wiz_SetOpts()
   if Wiz_SetOpts(dummy, self.DCList) then
      self.Memo.Lines.Add('Wiz_SetOpts succeeded')
   else
      self.Memo.Lines.Add('Error: Wiz_SetOpts failed');

   // note: this demo is currently not working and will cause
   // an access violation in unzip32.dll
end;


procedure TForm_Main.OnClick_Button_WizSingleEntryUnzip (Sender: TObject);
var
   incl:      array of PChar;
   excl:      array of PChar;
begin
   // demo for Wiz_SingleEntryUnzip()
   SetLength(incl, 2); incl[0] := '*.txt'; incl[1] := '*.jpg';   // unzip only txt's and jpg's
   SetLength(excl, 2); excl[0] := '*.url'; excl[1] := '*.exe';   // don't unzip url's and exe's

   // note: the declaration of both include and exclude file name vector is actually
   // redundant (all files that are not included are automatically excluded, and vice
   // versa). if you don't need any of these vectors, just pass empty PChar variables.
   // otherwise pass the first element ([0]) of your file vector array.

   self.Memo.Lines.Add(Wiz_ErrorToStr(Wiz_SingleEntryUnzip(2, incl[0], 2, excl[0], self.DCList, self.UFuncs)));
end;


procedure TForm_Main.OnClick_Button_WizUnzip (Sender: TObject);
var
   dummy:      Pointer;
   incl:       array of PChar;
   excl:       PChar;
begin
   // demo for Wiz_Unzip()
   SetLength(incl, 2); incl[0] := '*.txt'; incl[1] := '*.jpg';   // same as above
   excl := nil;                                                  // but no excludes this time

   // note: this demo is not working due to the broken Wiz_SetOpts function
   self.Memo.Lines.Add(Wiz_ErrorToStr(Wiz_Unzip(dummy, 2, incl[0], 0, excl)));
end;


procedure TForm_Main.OnClick_Button_WizUnzipToMemory(Sender: TObject);
var
   buffer:      UzpBuffer;
   r:           Longint;
begin
   // demo for Wiz_UnzipToMemory() and UzpFreeMemBuffer()
   buffer.strlength := 0;
   buffer.strptr := nil;

   r := Wiz_UnzipToMemory(PATH_TO_ZIPFILE, SOME_ZIPPED_FILE, self.UFuncs, buffer);

   // note: on successfull memory allocation and initialization,
   // Wiz_UnzipToMemory() returns a Longbool instead of a return code.
   // therefore the return value needs to be translated if < PK_BADERR. 

   if r = 1 then
      r := PK_OK
   else if r = 0 then
      r := PK_ERR;

   if r = PK_OK then
      self.Memo.Lines.Add(Wiz_ErrorToStr(r) + ' (' + IntToStr(buffer.strlength) + ' bytes in buffer)')
   else
      self.Memo.Lines.Add(Wiz_ErrorToStr(r));

   // note: don't forget to clean up when you're done with the buffer.
   // since it was allocated by the DLL, it also needs to be released by it.

   UzpFreeMemBuffer(buffer);
end;


procedure TForm_Main.OnClick_Button_WizValidate (Sender: TObject);
begin
   // demo for Wiz_Validate()
   self.Memo.Lines.Add(Wiz_ErrorToStr(Wiz_Validate(PATH_TO_ZIPFILE, 1)));
end;


procedure TForm_Main.OnClick_Button_UzpVersion (Sender: TObject);
var
   version:      PUzpVer;
begin
   // demo for UzpVersion()
   version := UzpVersion;

   if not Assigned(version) then
      self.Memo.Lines.Add('Error: UzpVersion failed')
   else
      Form_Version.ShowVersion(version^);
end;


procedure TForm_Main.OnClick_Button_UzpVersion2(Sender: TObject);
var
   version:      UzpVer2;
begin
   // demo for UzpVersion2()
   UzpVersion2(version);
   Form_Version.ShowVersion(version);
end;


{*
========================
   callback functions
========================
*}

procedure CallbackMessage (ucsize, csiz, cfactor, mo, dy, yr, hh, mm: Longword; c: Byte; fname, meth: PChar; crc: Longword; fCrypt: Byte);
begin Form_Main.Memo.Lines.Add(Format('unzip32 Message: %u%s%u  %u%s  %u-%u-%u  %u:%u  %s', [ucsize, '/', csiz, cfactor, '%', mo, dy, yr, hh, mm, fname])); end;


function CallbackPassword (pwbuf: PChar; size: Longint; m, efn: PChar): EDllPassword;
begin Result := IZ_PW_NONE; end;


function CallbackPrint (buffer: PChar; size: Longword): EDllPrint;
begin
   Form_Main.Memo.Lines.Add('unzip32 Print: ' + buffer);
   Result := size;
end;


function CallbackReplace (filename: PChar): EDllReplace;
begin Result := IDM_REPLACE_NONE; end;


function CallbackService (efn: PChar; details: Longword): EDllService;
begin Result := UZ_ST_CONTINUE; end;



end.
